/*
 * http://chipspace.ru/httpchipspace-rustm32-general-purpose-timers-3-pwm-stdperiph_lib/
 */

#include "stm32l1xx.h"

GPIO_InitTypeDef GPIO_InitStructure;
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;

uint16_t F_output = 1000;//Частота выходного сигнала, Гц (задается)
uint16_t duty_cycle = 10;//Коэффициент заполнения (duty cycle), от 0% до 100% (задается)
uint16_t AutoReload_Value = 0;//Значение для записи в регистр TIM2_ARR (определяет период сигнала PWM, вычисляется в программе)
uint16_t Pulse_Length = 0;//Значение для записи в регистр TIM2_CCR1 (определяет длительность импульса сигнала PWM, вычисляется в программе)

int main()
 {
  /*Обновляем значение переменной SystemCoreClock - системной тактовой частоты, понадобится при дальнейших вычислениях*/
  SystemCoreClockUpdate();

  //Инициализация порта
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE); //Включаем тактирование GPIOA

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;//Настраивать будем вывод PA0
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;//Выход, альтернативная функция
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;//Push-Pull - двухтактный
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;//Без подтягивающих резисторов
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;//Работает на максимальной частоте
  GPIO_Init(GPIOA, &GPIO_InitStructure);//Прописываем в регистры заданные значения
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_TIM2);//PA0 подключаем к TIM2

  //Инициализация таймера TIM2
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);//Включаем тактирование TIM2
  /*Расчет значения для регистра автоперезагрузки, которым задается периoд сигнала PWM*/
  AutoReload_Value = (uint16_t) (SystemCoreClock/F_output) - 1;
  //Расчет значения для регистра CCR1, которым задается длительность импульса
  Pulse_Length = (uint16_t) ((AutoReload_Value*duty_cycle)/100);

  //Инициализация модуля Time base unit
  /*Задаем коэффициенты деления внешних предделителей тактовой частоты.
  В данном случае делители не используются,
  входная частота таймера равна системной частоте*/
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;//Направление счета - инкремент
  /*Указываем, что в TIM2_ARR необходимо записать
  вычисленное значение для нужного периода сигнала*/
  TIM_TimeBaseStructure.TIM_Period = AutoReload_Value;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);//Записываем все заданные выше значения в регистры

 //Настройка канала Capture/Compare для формирования PWM сигнала на выходе
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;//Режим PWM1
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//Разрешаем подключение к выводу мк
 /*Указываем, что в TIM2_CCR1 необходимо записать
  вычисленное значение для нужной длительности импульса*/
  TIM_OCInitStructure.TIM_Pulse = Pulse_Length;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;//Положительная полярность
  TIM_OC1Init(TIM2, &TIM_OCInitStructure);//Записываем все заданные выше значения в регистры

  TIM_OC1PreloadConfig(TIM2, TIM_OCPreload_Enable);//Разрешаем предварительную загрузку регистра сравнения
  TIM_ARRPreloadConfig(TIM2, ENABLE);//Разрешаем предварительную загрузку регистра автоперезагрузки
  /* TIM2 enable counter */
  TIM_Cmd(TIM2, ENABLE);//Запускаем счет таймера

  while (1);
 }
